' File name: HZ-WUDA.BAS
' Description: Converter from HZ to Wuda (Big Five)

' Software: DOS QBASIC Converter for HZ and Wuda (Big Five)
' Version: 2.0
' Date: Gregorian 1998-06-22 (Mon), Northequinox 1998-04-03
' Author: ZHONG Qiyao, (HZ) ~{VSFtR"~}, (Wuda) ҳ

' File Set: HZ-WUDA.BAS, WUDA-HZ.BAS, README.TXT
' Copyright: 1995, 1998, ZHONG Qiyao

' ----------------------------------------------------------------------
' USAGE:
' Must define the HC.TAB to the convertion table (see below) .
' ----------------------------------------------------------------------

CONST HC.TAB = "c:\dosprog\hz\convert\table.txt"

' Do not change anything below

DECLARE SUB InitReadTable (f$)
DECLARE SUB FileFin ()
DECLARE FUNCTION FileInit% ()
DECLARE SUB AscLineLink ()
DECLARE SUB AscLineEnd ()
DECLARE SUB ChinLineEnd ()
DECLARE SUB ChinChar ()
DECLARE SUB LineEnd ()
DECLARE SUB Fin ()
DECLARE SUB LineParse ()
DECLARE FUNCTION Init% ()
DECLARE SUB AscToChin ()
DECLARE SUB AscTilde ()
DECLARE SUB ChinToAsc ()
DECLARE FUNCTION CaseAsc% ()
DECLARE FUNCTION CaseChin% ()
DECLARE SUB AscChar ()

'/*----------------------------------------------------------------------
'  Convert HZ to Big 5
'  by Daniel Chung
'  ----------------------------------------------------------------------*/

'/* State of machine while converting a line. */
'/* AM = ASCII, CH = Chinese */
CONST AM = 1
CONST CH = 2

'/* Font. */
CONST SYMBOL.BEGIN = 33         'ASC("!")
CONST SYMBOL.END = 42           'ASC(")")
CONST CHAR.BEGIN = 48           'ASC("0")
CONST CHAR.END = 96 + 23        'ASC("w")
CONST BYTE2.BEGIN = 33          'ASC("!")
CONST BYTE2.END = 126           'ASC("~")

'/*----------------------------------------------------------------------
'  main: Main.
'  ----------------------------------------------------------------------*/
DEFINT A-Z

DIM SHARED filename AS STRING * 80
DIM SHARED str AS STRING * 256          'current string of text
DIM SHARED strlen                       'len of non-empty part of text
DIM SHARED x                            'current position in string

DIM SHARED outfile AS STRING * 80
DIM SHARED keephoriz                    'whether to keep horiz position
DIM SHARED printfreq                    'whether use frequent char set
DIM SHARED table(96, 96) AS STRING * 2  'conversion table

'/* init */
IF Init = 0 THEN END
   
WHILE FileInit = 1

	WHILE (NOT EOF(1))
		LINE INPUT #1, str
		LineParse
	WEND
	FileFin
WEND

'/* fin */
Fin

END

'/*----------------------------------------------------------------------
'  asc_char: ASCII mode -- print character.
'  ----------------------------------------------------------------------*/
SUB AscChar

c = ASC(MID$(str, x, 1))

'/* Ignore high-order characters. */
IF (c >= 128) THEN
	PRINT #2, " ";

'/* Keep printable characters and control characters. */
ELSE
	PRINT #2, CHR$(c);
END IF

END SUB

SUB AscLineEnd

END SUB

SUB AscLineLink

END SUB

'/*----------------------------------------------------------------------
'  asc_tilde: ASCII mode -- print "~".
'  ----------------------------------------------------------------------*/
SUB AscTilde

IF keephoriz = 1 THEN
	PRINT #2, " ~";
ELSE
	PRINT #2, "~";
END IF

END SUB

'/*----------------------------------------------------------------------
'  asc_to_chin: ASCII mode -- change to Chinese mode.
'  ----------------------------------------------------------------------*/
SUB AscToChin

IF keephoriz = 1 THEN
	PRINT #2, "  ";
END IF

END SUB

'/*----------------------------------------------------------------------
'  case_asc: Finite state machine -- ASCII mode.
'
'  Return mode, or 0 for end-of-line.
'  ----------------------------------------------------------------------*/
FUNCTION CaseAsc

IF MID$(str, x, 2) = "~{" THEN          '/* AM -> CH */
	AscToChin
	x = x + 2
	CaseAsc = CH
	
ELSEIF MID$(str, x, 2) = "~~" THEN      '/* "~~" -> "~" */
	AscTilde
	x = x + 2
	CaseAsc = AM
					'/* link to next line */
ELSEIF MID$(str, x, 1) = "~" AND x = strlen THEN
	AscLineLink
	CaseAsc = 0

ELSEIF x >= strlen + 1 THEN             '/* end of line */
	AscLineEnd
	CaseAsc = 0
	
ELSE                                    '/* ASCII char */
	AscChar
	x = x + 1
	CaseAsc = AM

END IF
END FUNCTION

'*----------------------------------------------------------------------
' case_chin: Finite state machine -- Chinese mode.
' ----------------------------------------------------------------------*/
FUNCTION CaseChin

					'/* end of line */
IF MID$(str, x, 2) = "~}" AND x + 1 = strlen THEN
	ChinLineEnd
	CaseChin = 0

ELSEIF MID$(str, x, 2) = "~}" THEN      '/* CH -> AM */
	ChinToAsc
	x = x + 2
	CaseChin = AM
					'/* end of line */
ELSEIF (x >= strlen) THEN
	ChinLineEnd
	CaseChin = 0
	
ELSE                                    '/* CH char */
	ChinChar
	x = x + 2
	CaseChin = CH
END IF

END FUNCTION

'/*----------------------------------------------------------------------
'  chin_char: Chinese mode -- print character.
'  ----------------------------------------------------------------------*/
SUB ChinChar

a = ASC(MID$(str, x))
b = ASC(MID$(str, x + 1))

'/* Check valid */
IF ((a < SYMBOL.BEGIN) OR (a > CHAR.END)) THEN
	v = 0
ELSEIF (a > SYMBOL.END) AND (a < CHAR.BEGIN) THEN
	v = 0
ELSEIF (b < BYTE2.BEGIN) OR (b > BYTE2.END) THEN
	v = 0

ELSE
	v = 1
END IF

'/* print */
IF v = 1 THEN                                           'Valid HZ
	IF (ASC(table(a - 32, b - 32)) > 0) THEN
		PRINT #2, table(a - 32, b - 32);        'Big-Five character
	ELSE
		PRINT #2, CHR$(&HA1); CHR$(&HBC);       'Square symbol
	END IF
ELSE                                                    'Invalid HZ
	PRINT #2, CHR$(&HA1); CHR$(&HBC);               'Square symbol
END IF

END SUB

SUB ChinLineEnd

END SUB

'/*----------------------------------------------------------------------
'  chin_to_asc: Chinese mode -- change to ASCII mode.
'  ----------------------------------------------------------------------*/
SUB ChinToAsc
     
IF keephoriz = 1 THEN
	PRINT #2, "  ";
END IF

END SUB

SUB FileFin

CLOSE #1
CLOSE #2

PRINT

END SUB

FUNCTION FileInit

'/* file */
INPUT "name of input file (. to end): ", filename

IF LEFT$(filename, 1) = "." THEN
	FileInit = 0
	GOTO FileInit.exit
END IF

OPEN filename FOR INPUT AS #1

'/* output */
INPUT "name of output file: ", outfile
OPEN outfile FOR OUTPUT AS #2

FileInit = 1

FileInit.exit:
END FUNCTION

SUB Fin

END SUB

'/*----------------------------------------------------------------------
'  init: Initialise.
'  ----------------------------------------------------------------------*/
FUNCTION Init

'/* parameters */
INPUT "keep horizontal positions (1 or 0): ", keephoriz
INPUT "prefer frequently-used character set (1 or 0): ", printfreq

'/* read table */
PRINT "reading conversion table ..."
InitReadTable (HC.TAB)
	
Init = 1

END FUNCTION

SUB InitReadTable (f$)

OPEN f$ FOR INPUT AS #3
WHILE NOT EOF(3)
	LINE INPUT #3, s$
	IF MID$(s$, 1, 1) <> "#" THEN
		a = ASC(MID$(s$, 3, 1)) - 32
		b = ASC(MID$(s$, 4, 1)) - 32
		IF ASC(table(a, b)) = 0 THEN table(a, b) = MID$(s$, 7, 2)
	      
		'less-often character set?
		IF printfreq = 1 AND ASC(table(a, b)) > 200 THEN
			FOR i = 7 TO LEN(s$) - 1 STEP 2
				IF ASC(MID$(s$, i, 1)) <= 200 THEN
					table(a, b) = MID$(s$, i, 2)
					GOTO InitReadTable.brk
				END IF
			NEXT i
		END IF
InitReadTable.brk:
	END IF
WEND

CLOSE #3

GOTO skip

'/* find what characters have no correspondence */
OPEN "hzonly.txt" FOR OUTPUT AS #4

FOR i = SYMBOL.BEGIN TO SYMBOL.END
	FOR j = BYTE2.BEGIN TO BYTE2.END
		a = i - 32
		b = j - 32
		IF ASC(table(a, b)) = 0 THEN
			PRINT #4, "~{"; CHR$(i); CHR$(j); "~}~"
		END IF
	NEXT
NEXT

FOR i = CHAR.BEGIN TO CHAR.END
	FOR j = BYTE2.BEGIN TO BYTE2.END
		a = i - 32
		b = j - 32
		IF ASC(table(a, b)) = 0 THEN
			PRINT #4, "~{"; CHR$(i); CHR$(j); "~}~"
		END IF
	NEXT
NEXT

CLOSE #4

skip:

END SUB

SUB LineEnd

PRINT ".";

PRINT #2, ""

END SUB

'*----------------------------------------------------------------------
' line_print: Finite state machine for printing line.
' ----------------------------------------------------------------------*/
SUB LineParse

x = 1
s = AM                     'local variable: "state"

'/* string length */
strlen = 0
FOR i = LEN(str) TO 1 STEP -1
	IF MID$(str, i, 1) <> " " THEN
		strlen = i
		GOTO brk
	END IF
NEXT i

brk:

'/* loop */
WHILE s <> 0

	SELECT CASE s
	CASE AM
		s = CaseAsc
	CASE CH
		s = CaseChin
	END SELECT
WEND

LineEnd

END SUB

