' File name: WUDA-HZ.BAS
' Description: Converter from Wuda (Big Five) to HZ

' Software: DOS QBASIC Converter for HZ and Wuda (Big Five)
' Version: 2.0
' Date: Gregorian 1998-06-22 (Mon), Northequinox 1998-04-03
' Author: ZHONG Qiyao, (HZ) ~{VSFtR"~}, (Wuda) ҳ

' File Set: HZ-WUDA.BAS, WUDA-HZ.BAS, README.TXT
' Copyright: 1995, 1998, ZHONG Qiyao

' ----------------------------------------------------------------------
' USAGE:
' Must define the HC.TAB to the convertion table (see below) .
' ----------------------------------------------------------------------

CONST HC.TAB = "c:\dosprog\hz\convert\table.txt"

' Do not change anything below

DECLARE SUB InitReadTable (f$)
DECLARE SUB FileFin ()
DECLARE FUNCTION FileInit% ()
DECLARE SUB AscLineEnd ()
DECLARE SUB ChinLineEnd ()
DECLARE SUB ChinChar ()
DECLARE SUB LineEnd ()
DECLARE SUB Fin ()
DECLARE SUB LineParse ()
DECLARE FUNCTION Init% ()
DECLARE SUB AscToChin ()
DECLARE SUB AscTilde ()
DECLARE SUB ChinToAsc ()
DECLARE FUNCTION CaseAsc% ()
DECLARE FUNCTION CaseChin% ()
DECLARE SUB AscChar ()

'/*----------------------------------------------------------------------
'  Convert Big 5 to HZ
'  by Daniel Chung
'  ----------------------------------------------------------------------*/

'/* State of machine while converting a line. */
'/* AM = ASCII, CH = Chinese */
CONST AM = 1
CONST CH = 2

'/* Font. */
CONST BYTE1.BEGIN = &HA1
CONST BYTE1.END = &HFE
CONST BYTE2A.BEGIN = &H40
CONST BYTE2A.END = &H7E
CONST BYTE2B.BEGIN = &HA1
CONST BYTE2B.END = &HFE

'/*----------------------------------------------------------------------
'  main: Main.
'  ----------------------------------------------------------------------*/
DEFINT A-Z

DIM SHARED filename AS STRING * 80
DIM SHARED str AS STRING * 256          'current string of text
DIM SHARED strlen                       'len of non-empty part of text
DIM SHARED x                            'current position in string
DIM SHARED link, printlink              'whether link to next line

DIM SHARED outfile AS STRING * 80
DIM SHARED keephoriz                    'whether to keep horiz position

					'conversion table
DIM SHARED tableA(96, 64) AS STRING * 2
DIM SHARED tableB(96, 96) AS STRING * 2

'/* init */
IF Init = 0 THEN END
   
WHILE FileInit = 1

	WHILE (NOT EOF(1))
		LINE INPUT #1, str
		LineParse
	WEND
	FileFin
WEND

'/* fin */
Fin

END

'/*----------------------------------------------------------------------
'  asc_char: ASCII mode -- print character.
'  ----------------------------------------------------------------------*/
SUB AscChar

c = ASC(MID$(str, x, 1))

'/* Ignore high-order characters. */
IF (c >= 128) THEN
	PRINT #2, " ";

'/* Keep printable characters and control characters. */
ELSE
	PRINT #2, CHR$(c);
END IF

END SUB

SUB AscLineEnd

END SUB

'/*----------------------------------------------------------------------
'  asc_tilde: ASCII mode -- print "~".
'  ----------------------------------------------------------------------*/
SUB AscTilde

PRINT #2, "~~";

END SUB

'/*----------------------------------------------------------------------
'  asc_to_chin: ASCII mode -- change to Chinese mode.
'  ----------------------------------------------------------------------*/
SUB AscToChin

PRINT #2, "~{";

END SUB

'/*----------------------------------------------------------------------
'  case_asc: Finite state machine -- ASCII mode.
'
'  Return mode, or 0 for end-of-line.
'  ----------------------------------------------------------------------*/
FUNCTION CaseAsc

					'/* AM -> CH */
IF MID$(str, x, 2) = "  " AND ASC(MID$(str, x + 2, 1)) >= 128 THEN
	IF keephoriz THEN
		AscToChin
		x = x + 2
		CaseAsc = CH
	ELSE
		AscChar
		x = x + 1
		CaseAsc = AM
	END IF

ELSEIF ASC(MID$(str, x, 1)) >= 128 THEN     '/* AM -> CH */
	AscToChin
	CaseAsc = CH
	
ELSEIF MID$(str, x, 2) = " ~" THEN      '/* " ~" -> "~~" */
	IF keephoriz THEN
		AscTilde
		x = x + 2
	ELSE
		AscChar
		x = x + 1
	END IF
	CaseAsc = AM

ELSEIF MID$(str, x, 1) = "~" THEN      '/* "~" -> "~~" */
	AscTilde
	x = x + 1
	CaseAsc = AM

ELSEIF x >= strlen + 1 THEN             '/* end of line */
	AscLineEnd
	CaseAsc = 0
	
ELSE                                    '/* ASCII char */
	AscChar
	x = x + 1
	CaseAsc = AM

END IF
END FUNCTION

'*----------------------------------------------------------------------
' case_chin: Finite state machine -- Chinese mode.
' ----------------------------------------------------------------------*/
FUNCTION CaseChin

					'/* CH -> AM */
IF MID$(str, x, 2) = "  " THEN
	ChinToAsc
	IF keephoriz THEN x = x + 2
	CaseChin = AM

ELSEIF ASC(MID$(str, x, 1)) < 128 THEN
	ChinToAsc
	CaseChin = AM

					'/* end of line */
ELSEIF (x >= strlen) THEN
	ChinLineEnd
	CaseChin = 0
	
ELSE                                    '/* CH char */
	ChinChar
	x = x + 2
	CaseChin = CH
END IF

END FUNCTION

'/*----------------------------------------------------------------------
'  chin_char: Chinese mode -- print character.
'  ----------------------------------------------------------------------*/
SUB ChinChar

A = ASC(MID$(str, x))
b = ASC(MID$(str, x + 1))

'/* Check valid */
IF ((A < BYTE1.BEGIN) OR (A > BYTE1.END)) THEN
	v = 0
ELSEIF (b < BYTE2A.BEGIN) OR (b > BYTE2B.END) THEN
	v = 0
ELSEIF (b > BYTE2A.END) AND (b < BYTE2A.BEGIN) THEN
	v = 0

ELSE
	v = 1
END IF

'/* print */
IF v = 1 AND b <= BYTE2A.END THEN               'Big-Five character
	c = A - BYTE1.BEGIN + 1
	d = b - BYTE2A.BEGIN + 1
	IF (ASC(tableA(c, d)) > 0) THEN
		PRINT #2, tableA(c, d);         'HZ character
	ELSE
		PRINT #2, "!u";                 'Square symbol
	END IF
	       
ELSEIF v = 1 AND b <= BYTE2B.END THEN           'Big-Five character
	c = A - BYTE1.BEGIN + 1
	d = b - BYTE2B.BEGIN + 1
	IF (ASC(tableB(c, d)) > 0) THEN
		PRINT #2, tableB(c, d);         'HZ character
	ELSE
		PRINT #2, "!u";                 'Square symbol
	END IF

ELSE                                            'Invalid Big-Five character
	PRINT #2, tableA(1, 1);                 'Square symbol
END IF

END SUB

SUB ChinLineEnd

END SUB

'/*----------------------------------------------------------------------
'  chin_to_asc: Chinese mode -- change to ASCII mode.
'  ----------------------------------------------------------------------*/
SUB ChinToAsc
    
PRINT #2, "~}";

END SUB

SUB FileFin

CLOSE #1
CLOSE #2

PRINT

END SUB

FUNCTION FileInit

'/* file */
INPUT "name of input file (. to end): ", filename

IF LEFT$(filename, 1) = "." THEN
	FileInit = 0
	GOTO FileInit.exit
END IF

OPEN filename FOR INPUT AS #1

'/* output */
INPUT "name of output file: ", outfile
OPEN outfile FOR OUTPUT AS #2

FileInit = 1

FileInit.exit:
END FUNCTION

SUB Fin

END SUB

'/*----------------------------------------------------------------------
'  init: Initialise.
'  ----------------------------------------------------------------------*/
FUNCTION Init

'/* parameters */
INPUT "keep horizontal positions (1 or 0): ", keephoriz

'/* read table */
PRINT "reading conversion table ..."
InitReadTable (HC.TAB)

Init = 1

END FUNCTION

SUB InitReadTable (f$)

OPEN f$ FOR INPUT AS #3

WHILE NOT EOF(3)
	LINE INPUT #3, s$
	IF MID$(s$, 1, 1) <> "#" THEN

		'/* HZ */
		A = ASC(MID$(s$, 3, 1))
		b = ASC(MID$(s$, 4, 1))

		'/* dawu */
		FOR i = 7 TO LEN(s$) - 1 STEP 2
			c = ASC(MID$(s$, i, 1)) - BYTE1.BEGIN + 1
			d = ASC(MID$(s$, i + 1, 1))
			IF (d >= BYTE2A.BEGIN) AND (d <= BYTE2A.END) THEN
				d = d - BYTE2A.BEGIN + 1
				IF ASC(tableA(c, d)) = 0 THEN
					tableA(c, d) = CHR$(A) + CHR$(b)
				END IF
			ELSEIF (d >= BYTE2B.BEGIN) AND (d <= BYTE2B.END) THEN
				d = d - BYTE2B.BEGIN + 1
				IF ASC(tableB(c, d)) = 0 THEN
					tableB(c, d) = CHR$(A) + CHR$(b)
				END IF
			END IF
		NEXT i
	END IF
WEND

CLOSE #3

GOTO skip

'/* what character has Big-Five only */
OPEN "b5only.txt" FOR OUTPUT AS #4

FOR i = BYTE1.BEGIN TO BYTE1.END
	FOR j = BYTE2A.BEGIN TO BYTE2A.END
		c = i - BYTE1.BEGIN + 1
		d = j - BYTE2A.BEGIN + 1
		IF ASC(tableA(c, d)) = 0 THEN
			PRINT #4, CHR$(i); CHR$(j)
		END IF
	NEXT j
NEXT i

FOR i = BYTE1.BEGIN TO BYTE1.END
	FOR j = BYTE2B.BEGIN TO BYTE2B.END
		c = i - BYTE1.BEGIN + 1
		d = j - BYTE2B.BEGIN + 1
		IF ASC(tableB(c, d)) = 0 THEN
			PRINT #4, CHR$(i); CHR$(j)
		END IF
	NEXT j
NEXT i

CLOSE #4

skip:

END SUB

SUB LineEnd

PRINT ".";

PRINT #2, ""

END SUB

'*----------------------------------------------------------------------
' line_print: Finite state machine for printing line.
' ----------------------------------------------------------------------*/
SUB LineParse

x = 1
s = AM                     'local variable: "state"

'/* string length */
strlen = 0
FOR i = LEN(str) TO 1 STEP -1
	IF MID$(str, i, 1) <> " " THEN
		strlen = i
		GOTO brk
	END IF
NEXT i

brk:

'/* loop */
WHILE s <> 0

	SELECT CASE s
	CASE AM
		s = CaseAsc
	CASE CH
		s = CaseChin
	END SELECT
WEND

LineEnd

END SUB

